#!/usr/bin/python
#Rene Warren April 11th/2006
#
#Wrapper for the THOR web. app.
#-Launches the THOR java on specified server
#-Mails the user

import sys
import os
import commands
import time
import re
import glob

import smtplib
import MimeWriter
import base64
import StringIO
import string
import quopri

underscore_regex = re.compile('_')              ### underscore

RUN_DIR = sys.argv[1]
FROM_ADDR = ' "THOR" <thor@bcgsc.ca>'
TO_ADDR = [sys.argv[2],FROM_ADDR]
#TO_ADDR = [sys.argv[2],"rwarren@bcgsc.ca"]
#TO_ADDR = sys.argv[2]
ID = sys.argv[3]

ANIMAL = underscore_regex.sub(' ',sys.argv[4]) ### replace underscore by space

RESULT_FILE = sys.argv[1] + "COPA.fasta"
BLAST_FAIL = sys.argv[1] + "blastfail.names.out"
NO_READ = sys.argv[1] + "noreads.names.out"

FILE_NAME = "THOR.fasta"




########################################################
#CHECK ERROR FILE
########################################################
#error_file = glob.glob(RUN_DIR + '/*.e*')


########################################################
#EMAIL
########################################################


#### Read the output, email the user with the results
message = StringIO.StringIO()
writer = MimeWriter.MimeWriter(message)
writer.addheader('MIME-Version', '1.0')
writer.addheader('Subject', "Your THOR job id#%s\n" % (ID))
writer.addheader('To', sys.argv[2])
writer.addheader('From', FROM_ADDR)

# get ready to send attachment
writer.startmultipartbody('mixed')

# start off with a text/plain part
part = writer.nextpart()
body = part.startbody('text/plain')

body.write("Your THOR job id#%s (Assembly in %s) " % (ID, ANIMAL))




if not os.path.exists(RESULT_FILE):              ### no output given by COPA/THOR Java

   body.write("failed.\nSomething went wrong while running your job on the GSC-THOR server.\n\nPlease send your query again.\n\n")
   body.write("We apologize for the inconvenience,\n")
   body.write("--the THOR team")

else:                                            ### Output was produced, but results file could be empty.


   if os.path.getsize(RESULT_FILE) == 0 and os.path.getsize(BLAST_FAIL) == 0 and os.path.getsize(NO_READ) == 0:  ### this would indicate THOR failure

      body.write("failed.\nSomething went wrong while running your job on the GSC-THOR server.\n\nPlease send your query again.\n\n")
      body.write("We apologize for the inconvenience,\n")
      body.write("--the THOR team")


   else:                                         ### Program likely did not crash

      if os.path.getsize(NO_READ) == 0:          ### Found WGS reads     

         if os.path.getsize(BLAST_FAIL) == 0 and not os.path.getsize(RESULT_FILE) == 0:    ### Align in the right place

            body.write("completed without errors.  The results can be found in the attached file.\n\n")
            body.write("-the THOR team")

            part = writer.nextpart()
            part.addheader('Content-Transfer-Encoding', 'quoted-printable')

            f = open(RESULT_FILE, "rb")
            text = f.read()
            body = part.startbody('application/notepad; name=%s' % FILE_NAME)
            body.write(quopri.encodestring(text))
            f.close()

         else:                                   ### doesn't align in the right spot

            body.write("finished normally, but THOR could not find a suitable ortholog of your sequence.\n\nAn assembly was produced, but the reciprocal test used to assess the validity of the reconstruction failed.\nYou might want to try a different region or change the genome coordinates in the header of your fasta file.\n\n")
            body.write("-the THOR team")

      else:                                      ### could not fin WGS reads  

         body.write("finished normally, but THOR could not find a suitable ortholog of your sequence.\n\nTHOR could not recruit %s WGS reads for the assembly.\n\n" % ANIMAL)
         body.write("-the THOR team")



# finish off
writer.lastpart()

# send the mail
smtp = smtplib.SMTP('mail.bcgsc.ca', 25)
smtp.sendmail(FROM_ADDR, TO_ADDR, message.getvalue() )
smtp.quit()


